@@ -3,39 +3,34 @@ from __future__ import annotations
 import re
 import sys
 from datetime import datetime
-from typing import Union, Type, Optional, Callable, Awaitable
+from typing import Any, Awaitable, Callable, Type, cast
 from urllib.parse import urljoin
 
-from discord import (
-    Member,
-    Message,
-    RawMessageDeleteEvent,
-    VoiceState,
-    Guild,
-    Invite,
-    PartialEmoji,
-    Role,
-    Thread,
-)
+from discord import Guild, Invite, Member, Message, PartialEmoji, RawMessageDeleteEvent, Role, Thread, VoiceState
 from discord.abc import Messageable, User
-from discord.ext.commands import Cog as DiscordCog, Bot, Context, CommandError
+from discord.ext.commands.bot import Bot
+from discord.ext.commands.cog import Cog as DiscordCog
+from discord.ext.commands.context import Context
+from discord.ext.commands.core import Command
+from discord.ext.commands.errors import CommandError
 
-from PyDrocsid.config import Config, Contributor
+from PyDrocsid.config import Config
 from PyDrocsid.environment import DISABLED_COGS
-from PyDrocsid.events import register_events, event_handlers
+from PyDrocsid.events import event_handlers, register_events
 from PyDrocsid.logger import get_logger
 
+
 logger = get_logger(__name__)
 
 
 class Cog(DiscordCog):
-    CONTRIBUTORS: list[Contributor]
+    CONTRIBUTORS: list[tuple[int, str]]
     DEPENDENCIES: list[Type[Cog]] = []
 
-    instance: Optional[Cog] = None
+    instance: Cog | None = None
     bot: Bot
 
-    def __new__(cls, *args, **kwargs):
+    def __new__(cls, *args: Any, **kwargs: Any) -> Cog:
         """Make sure there exists only one instance of a cog."""
 
         if cls.instance is None:
@@ -62,82 +57,85 @@ class Cog(DiscordCog):
 
     # Event Handlers
 
-    async def on_ready(self):
+    async def on_ready(self) -> None:
         pass
 
-    async def on_typing(self, channel: Messageable, user: Union[User, Member], when: datetime):
+    async def on_typing(self, channel: Messageable, user: User | Member, when: datetime) -> None:
         pass
 
-    async def on_self_message(self, message: Message):
+    async def on_self_message(self, message: Message) -> None:
         pass
 
-    async def on_message(self, message: Message):
+    async def on_message(self, message: Message) -> None:
         pass
 
-    async def on_bot_ping(self, message: Message):
+    async def on_bot_ping(self, message: Message) -> None:
         pass
 
-    async def on_message_delete(self, message: Message):
+    async def on_message_delete(self, message: Message) -> None:
         pass
 
-    async def on_raw_message_delete(self, event: RawMessageDeleteEvent):
+    async def on_raw_message_delete(self, event: RawMessageDeleteEvent) -> None:
         pass
 
-    async def on_message_edit(self, before: Message, after: Message):
+    async def on_message_edit(self, before: Message, after: Message) -> None:
         pass
 
-    async def on_raw_message_edit(self, channel: Messageable, message: Message):
+    async def on_raw_message_edit(self, channel: Messageable, message: Message) -> None:
         pass
 
-    async def on_raw_reaction_add(self, message: Message, emoji: PartialEmoji, user: Union[Member, User]):
+    async def on_raw_reaction_add(self, message: Message, emoji: PartialEmoji, user: User | Member) -> None:
         pass
 
-    async def on_raw_reaction_remove(self, message: Message, emoji: PartialEmoji, user: Union[Member, User]):
+    async def on_raw_reaction_remove(self, message: Message, emoji: PartialEmoji, user: User | Member) -> None:
         pass
 
-    async def on_raw_reaction_clear(self, message: Message):
+    async def on_raw_reaction_clear(self, message: Message) -> None:
         pass
 
-    async def on_raw_reaction_clear_emoji(self, message: Message, emoji: PartialEmoji):
+    async def on_raw_reaction_clear_emoji(self, message: Message, emoji: PartialEmoji) -> None:
         pass
 
-    async def on_member_join(self, member: Member):
+    async def on_member_join(self, member: Member) -> None:
         pass
 
-    async def on_member_remove(self, member: Member):
+    async def on_member_remove(self, member: Member) -> None:
         pass
 
-    async def on_member_nick_update(self, before: str, after: str):
+    async def on_member_nick_update(self, before: str, after: str) -> None:
         pass
 
-    async def on_member_role_add(self, after, role: Role):
+    async def on_member_role_add(self, after: Member, role: Role) -> None:
         pass
 
-    async def on_member_role_remove(self, after, role: Role):
+    async def on_member_role_remove(self, after: Member, role: Role) -> None:
         pass
 
-    async def on_user_update(self, before: User, after: User):
+    async def on_user_update(self, before: User, after: User) -> None:
         pass
 
-    async def on_voice_state_update(self, member: Member, before: VoiceState, after: VoiceState):
+    async def on_voice_state_update(self, member: Member, before: VoiceState, after: VoiceState) -> None:
         pass
 
-    async def on_member_ban(self, guild: Guild, user: Union[User, Member]):
+    async def on_member_ban(self, guild: Guild, user: User | Member) -> None:
         pass
 
-    async def on_member_unban(self, guild: Guild, user: User):
+    async def on_member_unban(self, guild: Guild, user: User) -> None:
         pass
 
-    async def on_invite_create(self, invite: Invite):
+    async def on_invite_create(self, invite: Invite) -> None:
         pass
 
-    async def on_invite_delete(self, invite: Invite):
+    async def on_invite_delete(self, invite: Invite) -> None:
         pass
 
-    async def on_command_error(self, ctx: Context, error: CommandError):
+    async def on_command_error(self, ctx: Context[Bot], error: CommandError) -> None:
         pass
 
-    async def on_thread_join(self, thread: Thread):
+    async def on_thread_create(self, thread: Thread) -> None:
+        pass
+
+    async def on_thread_join(self, thread: Thread) -> None:
         pass
 
 
@@ -156,8 +154,8 @@ def check_dependencies(cogs: list[Cog]) -> set[Type[Cog]]:
     # required_by maps cog x to a list of cogs that depend on x
     required_by: dict[Type[Cog], list[Cog]] = {}
     for cog in cogs:
-        for dependency in cog.DEPENDENCIES:
-            required_by.setdefault(dependency, []).append(cog)
+        for dep in cog.DEPENDENCIES:
+            required_by.setdefault(dep, []).append(cog)
 
     # set of disabled cogs
     disabled: set[Type[Cog]] = set()
@@ -195,7 +193,7 @@ def check_dependencies(cogs: list[Cog]) -> set[Type[Cog]]:
     return disabled
 
 
-def register_cogs(bot: Bot, *cogs: Cog):
+def register_cogs(bot: Bot, *cogs: Cog) -> None:
     """Add cogs to the bot."""
 
     register_events(bot)
@@ -205,7 +203,7 @@ def register_cogs(bot: Bot, *cogs: Cog):
 
         # iterate over attributes of cog to find event handlers
         for e in dir(Cog):
-            func: Callable[..., Awaitable] = getattr(cog, e)
+            func: Callable[..., Awaitable[None]] = getattr(cog, e)
 
             # event handlers must differ from the default handler defined in Cog
             if e.startswith("on_") and callable(func) and getattr(type(cog), e) is not getattr(Cog, e):
@@ -221,10 +219,10 @@ def register_cogs(bot: Bot, *cogs: Cog):
                 break
 
             Config.CONTRIBUTORS.update(cls.CONTRIBUTORS)
-            Config.ENABLED_COG_PACKAGES.add(sys.modules[cls.__module__].__package__)
+            Config.ENABLED_COG_PACKAGES.add(cast(str, sys.modules[cls.__module__].__package__))
 
 
-def load_cogs(bot: Bot, *cogs: Cog):
+def load_cogs(bot: Bot, *cogs: Cog) -> None:
     """Load and prepare cogs, resolve dependencies and add cogs to the bot."""
 
     disabled_cogs: list[Cog] = []
@@ -248,9 +246,9 @@ def load_cogs(bot: Bot, *cogs: Cog):
 
     if bot.cogs:
         logger.info("\033[1m\033[32m%s Cog%s enabled:\033[0m", len(bot.cogs), "s" * (len(bot.cogs) > 1))
-        for cog in bot.cogs.values():
-            commands = ", ".join(cmd.name for cmd in cog.get_commands())
-            logger.info(" + %s %s", cog.__class__.__name__, commands and f"({commands})")
+        for _cog in bot.cogs.values():
+            commands = ", ".join(cmd.name for cmd in cast(list[Command[Cog, Any, Any]], _cog.get_commands()))
+            logger.info(" + %s %s", _cog.__class__.__name__, commands and f"({commands})")
 
     if disabled_cogs:
         logger.info("\033[1m\033[31m%s Cog%s disabled:\033[0m", len(disabled_cogs), "s" * (len(disabled_cogs) > 1))
@@ -258,15 +256,12 @@ def load_cogs(bot: Bot, *cogs: Cog):
             logger.info(" - %s", name.__class__.__name__)
 
 
-def get_documentation(cog: Union[Cog, Type[Cog]]) -> Optional[str]:
+def get_documentation(cog: Cog | Type[Cog]) -> str | None:
     if isinstance(cog, Cog):
         cog = type(cog)
 
     for cls in cog.mro():
-        if match := re.match(
-            r"^cogs\.[a-zA-Z\d\-_]+\.([a-zA-Z\d\-_]+)\.([a-zA-Z\d\-_]+)\.cog$",
-            cls.__module__,
-        ):
+        if match := re.match(r"^cogs\.[a-zA-Z\d\-_]+\.([a-zA-Z\d\-_]+)\.([a-zA-Z\d\-_]+)\.cog$", cls.__module__):
             return urljoin(Config.DOCUMENTATION_URL, f"cogs/{match.group(1)}/{match.group(2)}/")
 
     return None
